# Generated by Django 2.2.28 on 2022-10-09 20:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('minidebconf', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Diet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('option', models.CharField(max_length=32, verbose_name='Diet')),
            ],
        ),
        migrations.CreateModel(
            name='ShirtSize',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=32, verbose_name='Description')),
            ],
        ),
        migrations.AlterModelOptions(
            name='registration',
            options={'verbose_name': 'registration', 'verbose_name_plural': 'registrations'},
        ),
        migrations.AlterField(
            model_name='registration',
            name='gender',
            field=models.CharField(blank=True, choices=[('', 'Decline to state'), ('m', 'Male'), ('f', 'Female'), ('n', 'Non-Binary')], max_length=1, null=True, verbose_name='Gender'),
        ),
        migrations.AddField(
            model_name='registration',
            name='diet',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='minidebconf.Diet'),
        ),
        migrations.AddField(
            model_name='registration',
            name='shirt_size',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='minidebconf.ShirtSize'),
        ),
    ]



# Generated by Django 4.2.10 on 2024-02-16 16:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("minidebconf", "0008_alter_registration_shirt_size"),
    ]

    operations = [
        migrations.AlterField(
            model_name="registration",
            name="diet",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="minidebconf.diet",
                verbose_name="Diet",
            ),
        ),
        migrations.AlterField(
            model_name="registration",
            name="registration_type",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="minidebconf.registrationtype",
            ),
        ),
        migrations.AlterField(
            model_name="registration",
            name="shirt_size",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="minidebconf.shirtsize",
                verbose_name="Shirt size",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-09 02:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0002_separate_childcare'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='accomm',
            name='special_needs',
        ),
    ]



from register.dates import parse_date
from register.forms.accommodation import AccommodationForm
from register.models.accommodation import Accomm, AccommNight
from register.models.queue import Queue
from register.views.core import RegisterStep


class AccommodationView(RegisterStep):
    template_name = 'register/page/accommodation.html'
    title = 'Accommodation'
    form_class = AccommodationForm

    def get_initial(self):
        user = self.request.user
        initial = {}

        try:
            accomm = user.attendee.accomm
        except Accomm.DoesNotExist:
            return initial

        for field in accomm._meta.get_fields():
            if field.is_relation:
                continue
            initial[field.name] = getattr(accomm, field.name)

        initial['accomm'] = True
        initial['nights'] = [night.form_name for night in accomm.nights.all()]

        return initial

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['attendee'] = self.request.user.attendee
        return kwargs

    def form_valid(self, form):
        attendee = self.request.user.attendee
        data = form.cleaned_data.copy()

        nights = data.pop('nights')
        if not data.pop('accomm'):
            Accomm.objects.filter(attendee=attendee).delete()
            return super().form_valid(form)

        accomm = Accomm.objects.update_or_create(
            attendee=attendee, defaults=data)[0]
        attendee.accomm = accomm

        stored_nights = set(accomm.nights.all())
        requested_nights = set()
        for night in nights:
            date = parse_date(night.split('_')[1])
            requested_nights.add(AccommNight.objects.get(date=date))

        accomm.nights.remove(*(stored_nights - requested_nights))
        accomm.nights.add(*(requested_nights - stored_nights))

        return super().form_valid(form)

    def registration_completed(self, attendee):
        try:
            accomm = attendee.accomm
        except Accomm.DoesNotExist:
            accomm = None
        if accomm:
            queue = Queue.objects.get_or_create(name='Accommodation')[0]
            queue.slots.get_or_create(attendee=attendee)



class TestHome:
    def test_visitor(self, client):
        client.logout()
        res = client.get("/")
        assert res.status_code == 200

    def test_logged_in(self, client):
        res = client.get("/")
        assert res.status_code == 200

    def test_superuser(self, user, client):
        user.is_staff = user.is_superuser = True
        user.save()
        res = client.get("/")
        assert res.status_code == 200



from register.forms.covid19 import Covid19Form
from register.views.core import RegisterStep
from register.models.attendee import Attendee


class Covid19View(RegisterStep):
    title = 'COVID-19'
    form_class = Covid19Form
    template_name = 'register/page/covid19.html'

    def get_initial(self):
        attendee = self.request.user.attendee
        initial = {
            'vaccinated': attendee.vaccinated,
            'vaccination_notes': attendee.vaccination_notes,
            'confirm_covid_tests': attendee.confirm_covid_tests,
        }
        return initial

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data

        user.attendee  # We should never be creating, here
        user.attendee = Attendee.objects.update_or_create(
            user=user, defaults=data)[0]

        return super().form_valid(form)



from django.contrib.auth import get_user_model
from django.test import Client
from django.utils import timezone

from pytest import fixture

from wafer.talks.models import Talk


User = get_user_model()


@fixture()
def disable_caching(settings):
    settings.CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        }
    }


@fixture
def user(db):
    user = User.objects.create(
        username="theuser",
        first_name="Firstname",
        last_name="Lastname",
    )
    yield user
    user.delete()


@fixture
def create_talk(db):
    talks = []
    users = []
    def _create_talk(username="author", title="Hello World", status='S'):
        author, created = User.objects.get_or_create(username=username)
        if created:
            users.append(author)
        talk = Talk.objects.create(
            title=title,
            status=status,
            corresponding_author=author)
        talk.authors.set([author])
        talks.append(talk)
        return talk

    yield _create_talk

    for talk in talks:
        talk.delete()
    for user in users:
        user.delete()


@fixture
def talk(create_talk):
    return create_talk()


@fixture
def client(db, user):
    _client = Client()
    _client.force_login(user)
    return _client


@fixture()
def now(mocker):
    n = timezone.now()
    mocker.patch("django.utils.timezone.now", return_value=n)
    return n



from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.contrib.sites.models import Site
from django.db.models import Q
from django.template import Context, Template

from bursary.models import Bursary
from invoices.prices import invoice_user


SUBJECT = '{{ WAFER_CONFERENCE_NAME }}: Invoice Created'
TXT = '''Dear {{ name }},

We're sorry that we were unable to grant you food and/or accommodation
assistance for {{ WAFER_CONFERENCE_NAME }}.

We have now issued an invoice for your requested food and/or accommodation.
The invoice details are below.

If you still intend to attend the conference, please go to
<https://{{ WAFER_CONFERENCE_DOMAIN }}/accounts/profile/>
and pay the invoice directly online.

While we do accept cash payment on-site, we would strongly prefer if you
settled the invoice directly online.

If you wish to find your own food and accommodation, and attend the conference
itself without cost, please go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/>
and update your registration to remove any food or accommodation that you are
being invoiced for.
Your invoice will be cancelled as part of the registration process.

If you no longer wish to attend {{ WAFER_CONFERENCE_NAME }}, please go to:
<https://{{ WAFER_CONFERENCE_DOMAIN }}/register/unregister>
and unregister.

Invoice number #{{ invoice_number }}

{{ invoice_details }}

Thanks for your interest in attending,

The {{ WAFER_CONFERENCE_NAME }} Registration Team
'''


class Command(BaseCommand):
    help = ("Issue invoices for attendees who applied for bursaries but "
            "didn't receive them")

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')
        parser.add_argument('--site', type=int, default=1,
                            help='Django site ID, default: 1')

    def badger(self, user, invoice, dry_run, site):
        context = Context({
            'invoice_number': invoice.reference_number,
            'invoice_details': invoice.text_details(),
            'name': user.userprofile.display_name(),
            'WAFER_CONFERENCE_DOMAIN': site.domain,
            'WAFER_CONFERENCE_NAME': site.name,
        })

        txt = Template(TXT).render(context)
        subject = Template(SUBJECT).render(context)
        to = user.email
        if dry_run:
            print('I would badger:', to)
            return
        email_message = EmailMultiAlternatives(subject, txt, to=[to])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        site = Site.objects.get(id=1)
        for bursary in Bursary.objects.filter(
                Q(request_food=True,
                  food_status__in=('expired', 'denied', 'canceled'))
                | Q(request_accommodation=True,
                  accommodation_status__in=('expired', 'denied', 'canceled'))):
            user = bursary.user
            if not bursary.user.userprofile.is_registered():
                continue
            attendee = user.attendee
            if attendee.new_invoices.exists():
                continue
            if attendee.billable() and not attendee.paid():
                invoice = invoice_user(user, save=not dry_run)['invoice']
                self.badger(user, invoice, dry_run, site)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-07-16 12:50
from __future__ import unicode_literals

from django.db import migrations

from register.views import STEPS


def backfill_registration_timestamps(apps, schema_editor):
    last_step = len(STEPS) - 1
    Attendee = apps.get_model('register', 'Attendee')

    for attendee in Attendee.objects.filter(
            completed_register_steps=last_step,
            completed_timestamp__isnull=True):
        # Avoid using .save() so we don't touch updated_timestamp
        Attendee.objects.filter(id=attendee.id).update(
            completed_timestamp=attendee.updated_timestamp,
        )


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('register', '0008_backfill_registration_timestamps'),
    ]

    operations = [
        migrations.RunPython(backfill_registration_timestamps, noop),
    ]



from datetime import timedelta

from django.db import models
from django.db.models import Q
from django.db.models.signals import m2m_changed, post_save
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.template.defaultfilters import date

from wafer.schedule.models import ScheduleItem, Venue
from pytz import common_timezones

from volunteers.utils import get_start_end_for_scheduleitem

LATE_SIGNUP_LEEWAY = timedelta(minutes=10)


def tz_choices():
    timezones = common_timezones
    if settings.TIME_ZONE not in timezones:
        timezones.insert(0, settings.TIME_ZONE)
    return [(tz, tz) for tz in timezones]


class Volunteer(models.Model):

    RATINGS = (
            (0, 'No longer welcome'),
            (1, 'Poor'),
            (2, 'Not great'),
            (3, 'Average'),
            (4, 'Good'),
            (5, 'Superb'),
    )

    user = models.OneToOneField(settings.AUTH_USER_MODEL,
                                on_delete=models.CASCADE,
                                related_name='volunteer')

    preferred_categories = models.ManyToManyField('TaskCategory', blank=True)
    preferred_task_types = models.ManyToManyField('TaskTemplate', blank=True)

    staff_rating = models.IntegerField(null=True, blank=True, choices=RATINGS)
    staff_notes = models.TextField(null=True, blank=True)

    timezone = models.TextField(max_length=32, default=settings.TIME_ZONE,
                                choices=tz_choices())

    @property
    def annotated_tasks(self):
        return self.tasks.annotate_all()

    def __str__(self):
        return u'%s' % self.user


class AbstractTaskTemplate(models.Model):
    class Meta:
        abstract = True

    MANDATORY_TASK_FIELDS = [
        'name', 'description', 'nbr_volunteers_min', 'nbr_volunteers_max',
    ]
    TASK_TEMPLATE_FIELDS = MANDATORY_TASK_FIELDS + ['category']

    name = models.CharField(max_length=1024, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    category = models.ForeignKey('TaskCategory', blank=True, null=True,
                                 on_delete=models.PROTECT)
    required_permission = models.CharField(
        max_length=255, blank=True, null=True)

    nbr_volunteers_min = models.IntegerField(default=None, blank=True, null=True)
    nbr_volunteers_max = models.IntegerField(default=None, blank=True, null=True)

    def clean_fields(self, exclude=None):
        if exclude is None:
            exclude = []

        errors = {}

        try:
            super().clean_fields(exclude)
        except ValidationError as exc:
            errors = exc.error_dict

        for field in self.MANDATORY_TASK_FIELDS:
            if field not in exclude and not getattr(self, field):
                setattr(self, field, None)

        # Only keep fields that have been overridden from the template
        if hasattr(self, 'template') and self.template:
            for field in self.TASK_TEMPLATE_FIELDS:
                if (field not in exclude and
                        getattr(self.template, field) == getattr(self, field)):
                    setattr(self, field, None)

        # If TaskTemplate or Task without template, check mandatory fields
        if not hasattr(self, 'template') or not self.template:
            for field in self.MANDATORY_TASK_FIELDS:
                if field not in exclude and not getattr(self, field):
                    errors[field] = ['Your task needs a %s' % field]

        if errors:
            raise ValidationError(errors)


class TaskTemplate(AbstractTaskTemplate):
    """a template for a Task"""
    video_task = models.BooleanField(default=False)

    def __str__(self):
        return u'Template for %s' % self.name

    class Meta(AbstractTaskTemplate.Meta):
        permissions = (
            ("accept_video_tasks", "Can accept video tasks"),
            ("accept_fd_tasks", "Can accept front desk tasks"),
        )


class TaskQuerySet(models.QuerySet):
    @staticmethod
    def coalesce_from_template(field):
        return models.Func(
            models.F(field),
            models.F('template__%s' % field),
            function='coalesce',
        )

    def annotate_all(self):
        return self.select_related('template').annotate(
            nbr_volunteers=models.Count('volunteers'),
            min_volunteers=self.coalesce_from_template('nbr_volunteers_min'),
            max_volunteers=self.coalesce_from_template('nbr_volunteers_max'),
            name_=self.coalesce_from_template('name'),
            description_=self.coalesce_from_template('description'),
            category_=self.coalesce_from_template('category__name'),
        )


class Task(AbstractTaskTemplate):
    """Something to do.

    If the template is set, it will override the name, description and
    category fields.
    """
    class Meta(AbstractTaskTemplate.Meta):
        ordering = ['start', '-end', 'schedule_item', 'template__name', 'name']

    objects = TaskQuerySet.as_manager()

    venue = models.ForeignKey(
        Venue,
        null=True,
        blank=True,
        on_delete=models.PROTECT,
    )

    start = models.DateTimeField(blank=True)
    end = models.DateTimeField(blank=True)

    # Volunteers
    volunteers = models.ManyToManyField('Volunteer', related_name='tasks',
                                        blank=True)

    schedule_item = models.ForeignKey(
        ScheduleItem,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    template = models.ForeignKey(
        TaskTemplate,
        null=True,
        blank=True,
        on_delete=models.PROTECT,
    )

    def __str__(self):
        venue = self.get_venue()
        if venue:
            venue = venue.name
        return u'%s in %s (%s: %s - %s)' % (
            self.get_name(), venue, self.start.date(),
            self.start.time(), self.end.time())

    def nbr_volunteers(self):
        return self.volunteers.count()
    nbr_volunteers.short_description = "# reg'd"

    def datetime(self):
        return u'%s: %s - %s' % (date(self.start.date(), 'l, F d'),
                                 date(self.start.time(), 'H:i'),
                                 date(self.end.time(), 'H:i'))

    def get_name(self):
        if self.name:
            return self.name
        if self.template:
            return self.template.name
    get_name.short_description = 'Name'

    def get_description(self):
        if self.description:
            return self.description
        if self.template:
            return self.template.description
    get_description.short_description = 'Description'

    def get_category(self):
        if self.category:
            return self.category
        if self.template:
            return self.template.category
    get_category.short_description = 'Category'

    def get_nbr_volunteers_min(self):
        if self.nbr_volunteers_min:
            return self.nbr_volunteers_min
        if self.template:
            return self.template.nbr_volunteers_min
    get_nbr_volunteers_min.short_description = '# min'

    def get_nbr_volunteers_max(self):
        if self.nbr_volunteers_max:
            return self.nbr_volunteers_max
        if self.template:
            return self.template.nbr_volunteers_max
    get_nbr_volunteers_max.short_description = '# max'

    def get_required_permission(self):
        if self.required_permission:
            return self.required_permission
        if self.template:
            return self.template.required_permission
    get_required_permission.short_description = 'Permission'

    def get_venue(self):
        if self.schedule_item:
            return self.schedule_item.venue
        return self.venue
    get_venue.short_description = 'Venue'

    def clean_fields(self, exclude=None):
        if exclude is None:
            exclude = []
        if 'venue' in exclude:
            return super().clean_fields(exclude)

        errors = {}
        try:
            super().clean_fields(exclude)
        except ValidationError as exc:
            errors = exc.error_dict

        if 'venue' not in exclude and not self.get_venue():
            errors.setdefault('venue', []).append(_('Venue needs to be set'))
        if 'venue' not in exclude and self.schedule_item and self.venue:
            errors.setdefault('venue', []).append(_(
                'Venue cannot be set for a task attached to an Event'
            ))

        if self.schedule_item:
            self.start, self.end = get_start_end_for_scheduleitem(
                self.schedule_item
            )

        if 'start' not in exclude and not self.start:
            errors.setdefault('start', []).append(_('Tasks need a start time'))

        if 'end' not in exclude and not self.end:
            errors.setdefault('end', []).append(_('Tasks need an end time'))

        if errors:
            raise ValidationError(errors)

    def concurrent_tasks(self):
        return Task.objects.filter(
            (Q(start__lte=self.start) & Q(end__gt=self.start)) |
            (Q(start__gt=self.start) & Q(start__lt=self.end))
        ).exclude(id=self.id)

    @property
    def started(self):
        return self.start < timezone.now()

    @property
    def open_for_signup(self):
        return (self.nbr_volunteers < self.max_volunteers and
                self.end + LATE_SIGNUP_LEEWAY >= timezone.now())


class TaskCategory(models.Model):
    """ Category of a task, like: cleanup, moderation, etc. """

    class Meta:
        verbose_name = _('task category')
        verbose_name_plural = _('task categories')

    name = models.CharField(max_length=1024)
    description = models.TextField()

    def __str__(self):
        return self.name


def update_video_tasks(sender, **kwargs):
    if kwargs.get('action', 'post_add') != 'post_add':
        return
    schedule_item = kwargs['instance']

    try:
        start, end = get_start_end_for_scheduleitem(schedule_item)
    except ValueError:
        # slot didn't have a start time, ignore
        return

    for task in Task.objects.filter(schedule_item=schedule_item).all():
        modified = False
        if task.start != start or task.end != end:
            modified = True
            task.start = start
            task.end = end
            task.volunteers.clear()
        if task.venue != schedule_item.venue:
            modified = True
            task.venue = schedule_item.venue
        if modified:
            task.save()


post_save.connect(update_video_tasks, sender=ScheduleItem)
m2m_changed.connect(update_video_tasks, sender=ScheduleItem.slots.through)



from django.conf import settings
from django.forms import Form

from register.views.core import RegisterStep
from register.models.accommodation import PAID_ACCOMMODATION


class InstructionsView(RegisterStep):
    template_name = "register/page/instructions.html"
    title = 'Instructions'
    form_class = Form  # Square peg in a round hole

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            'DEBCONF_BURSARY_DEADLINE': settings.DEBCONF_BURSARY_DEADLINE,
            'DEBCONF_CONFIRMATION_DEADLINE':
                settings.DEBCONF_CONFIRMATION_DEADLINE,
            'DEBCONF_REVIEW_FREE_ATTENDEES':
                settings.DEBCONF_REVIEW_FREE_ATTENDEES,
            'PAID_ACCOMMODATION': PAID_ACCOMMODATION,
        })
        return context



from django.contrib import admin
from django.utils.html import format_html, format_html_join
from django.utils.translation import gettext_lazy as _
from django.db.models import Count, F, Q

from volunteers.models import Volunteer, Task, TaskCategory, TaskTemplate


def duplicate(modeladmin, request, queryset):
    for object in queryset:
        object.id = None
        object.save()


duplicate.short_description = "Duplicate selected record"


class DayListFilter(admin.SimpleListFilter):
    title = _('day')
    parameter_name = 'day'

    def lookups(self, request, model_admin):
        return (
            (1, 'Sunday'),
            (2, 'Monday'),
            (3, 'Tuesday'),
            (4, 'Wednesday'),
            (5, 'Thursday'),
            (6, 'Friday'),
            (7, 'Saturday'),
        )

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(start__week_day=self.value())


class NumTasksListFilter(admin.SimpleListFilter):
    title = _('tasks')
    parameter_name = 'tasks'

    def lookups(self, request, model_admin):
        return (
            (0, _('No tasks')),
            (1, _('1 - 5 tasks')),
            (2, _('More than 5 tasks')),
        )

    def queryset(self, request, queryset):
        query = queryset.annotate(num_tasks=Count('tasks'))

        if self.value() == '0':
            return query.filter(num_tasks__lte=0)
        if self.value() == '1':
            return query.filter(num_tasks__gte=1, num_tasks__lte=5)
        if self.value() == '2':
            return query.filter(num_tasks__gte=6)


class HasVolunteersListFilter(admin.SimpleListFilter):
    title = _('volunteers')
    parameter_name = 'volunteers'

    def lookups(self, request, model_admin):
        return (
            ('full', _('No more needed')),
            ('some', _('More needed')),
            ('none', _('None')),
        )

    def queryset(self, request, queryset):
        query = queryset.annotate(nbr_volunteers=Count('volunteers'))

        if self.value() == 'full':
            return query.filter(nbr_volunteers=F('nbr_volunteers_max'))
        elif self.value() == 'some':
            return query.filter(nbr_volunteers__lt=F('nbr_volunteers_max'))
        elif self.value() == 'none':
            return query.filter(nbr_volunteers=0)


class CategoryFilter(admin.SimpleListFilter):
    title = _('Category')
    parameter_name = 'category'

    def lookups(self, request, model_admin):
        return [(cat.pk, cat.name) for cat in TaskCategory.objects.all()]

    def queryset(self, request, queryset):
        pk = self.value()
        return queryset.filter(
            Q(category__pk=pk) | Q(template__category__pk=pk)
        )


class VolunteerAdmin(admin.ModelAdmin):
    fieldsets = (
        ('Personal information', {
            'fields': ('user', 'full_name', 'email', 'contact_number', 'timezone'),
        }),
        (None, {
            'fields': ('preferred_categories',),
        }),
        ('For staff', {
            'classes': ('collapse',),
            'fields': ('staff_rating', 'staff_notes'),
        }),
    )

    list_display = ('user', 'full_name', 'email', 'contact_number',
                    'num_tasks', 'staff_rating')
    list_editable = ('staff_rating',)
    list_filter = ('staff_rating', NumTasksListFilter)

    readonly_fields = ('full_name', 'email', 'contact_number', 'num_tasks')

    def full_name(self, volunteer):
        return u'%s %s' % (volunteer.user.first_name, volunteer.user.last_name)

    def email(self, volunteer):
        return volunteer.user.email

    def contact_number(self, volunteer):
        return volunteer.user.userprofile.contact_number

    def num_tasks(self, volunteer):
        return volunteer.tasks.count()


class TaskAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('template',),
        }),
        ('Task data', {
            'fields': ('category', 'name', 'description'),
        }),
        ('Space-time considerations', {
            'fields': ('schedule_item', 'venue', 'start', 'end'),
        }),
        ('Volunteers', {
            'fields': ('volunteers',
                       'nbr_volunteers_min', 'nbr_volunteers_max'),
        }),
    )

    list_display = (
        'get_name', 'template', 'start', 'end', 'venue',
        'nbr_volunteers', 'get_nbr_volunteers_min', 'get_nbr_volunteers_max',
        'volunteers_', 'get_category', 'schedule_item',
    )
    list_editable = ('venue', 'template')
    list_filter = (CategoryFilter, DayListFilter, HasVolunteersListFilter)
    search_fields = ('name', 'template__name', 'description',
                     'template__description')

    actions = [duplicate]

    def volunteers_(self, instance):
        return format_html('<ul>{0}</ul>', format_html_join(
            '',
            '<li>{}</li>',
            ((volunteer.user.get_full_name(),)
             for volunteer in instance.volunteers.all())
        ))
    volunteers_.short_description = 'Volunteers'


class TaskCategoryAdmin(admin.ModelAdmin):
    list_display = ('name', 'description')


class TaskTemplateAdmin(admin.ModelAdmin):
    list_display = ('name', 'nbr_volunteers_min', 'nbr_volunteers_max',
                    'video_task')


admin.site.register(Volunteer, VolunteerAdmin)
admin.site.register(Task, TaskAdmin)
admin.site.register(TaskCategory, TaskCategoryAdmin)
admin.site.register(TaskTemplate, TaskTemplateAdmin)



from django.conf import settings
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from minidebconf.models import Diet, Registration, RegistrationType, ShirtSize


class RegistrationAdmin(admin.ModelAdmin):
    readonly_fields = ["name"]

    @admin.display(description=_("Full name"))
    def name(self, registration):
        return registration.user.get_full_name()

    @admin.display(description=_("Involvement"))
    def _involvement(self, registration):
        return registration.get_involvement_display()

    @admin.display(boolean=True, description=_("Accomodation"))
    def accommodation(self, registration):
        return registration.arranged_accommodation

    @admin.display(boolean=True, description=_("Food"))
    def food(self, registration):
        return registration.arranged_food

    @admin.display(boolean=True, description=_("Travel"))
    def travel(self, registration):
        return registration.travel_reimbursement

    def get_fields(self, request, obj=None):
        return ["name"] + super().get_fields(request, obj)

    def get_list_display(self, request):
        __list_display__ = (
            "name",
            "country",
            "phone_number" if settings.MINIDEBCONF_REGISTER_PHONE else None,
            "registration_type" if RegistrationType.objects.count() > 1 else None,
            "_involvement",
            "gender",
            "country",
            "city_state",
            "diet" if Diet.objects.count() > 1 else None,
            "shirt_size" if ShirtSize.objects.count() > 1 else None,
            "accommodation",
            "food",
            "travel",
            "travel_cost",
        )
        return [f for f in __list_display__ if f]

    list_filter = (
        "registration_type",
        "involvement",
        "gender",
        "days",
        "diet",
        "shirt_size",
        "arranged_accommodation",
        "arranged_food",
        "travel_reimbursement",
    )


admin.site.register(Registration, RegistrationAdmin)
admin.site.register(RegistrationType)
admin.site.register(Diet)
admin.site.register(ShirtSize)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-20 15:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_countries.fields
import minidebconf.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('schedule', '0010_cleanup_old_data'),
    ]

    operations = [
        migrations.CreateModel(
            name='Registration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('involvement', models.IntegerField(blank=True, choices=[(0, 'Beginner'), (1, 'User'), (2, 'Contributor'), (4, 'Debian Maintainer (DM)'), (5, 'Debian Developer (DD)')], null=True, verbose_name='Level of involvement with Debian')),
                ('gender', models.CharField(blank=True, choices=[('m', 'Male'), ('f', 'Female'), ('n', 'Non-Binary')], max_length=1, null=True, verbose_name='Gender')),
                ('country', django_countries.fields.CountryField(blank=True, countries=minidebconf.models.Countries, max_length=2, null=True, verbose_name='Country')),
                ('city_state', models.CharField(blank=True, max_length=128, verbose_name='City/State or Province')),
                ('days', models.ManyToManyField(to='schedule.ScheduleBlock', verbose_name='Which days you will attend')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.contrib import admin
from django.db.models import Q


class NotNullFilter(admin.SimpleListFilter):
    title = 'Filter title not set'

    parameter_name = 'parameter name not set'

    def lookups(self, request, model_admin):

        return (
            ('not_null', 'Not empty only'),
            ('null', 'Empty only'),
        )

    def queryset(self, request, queryset):

        if self.value() == 'not_null':
            is_null_false = {
                self.parameter_name + "__isnull": False
            }
            exclude = {
                self.parameter_name: ""
            }
            return queryset.filter(**is_null_false).exclude(**exclude)

        if self.value() == 'null':
            is_null_true = {
                self.parameter_name + "__isnull": True
            }
            param_exact = {
                self.parameter_name + "__exact": ""
            }
            return queryset.filter(Q(**is_null_true) | Q(**param_exact))



import re

from django.core.management.base import BaseCommand

from wafer.talks.models import TalkUrl


class Command(BaseCommand):
    help = "Update etherpad URLs to point to archived TXT versions"

    def add_arguments(self, parser):
        parser.add_argument('--talk', type=int,
            help='Act only on a specific talk ID. Default: all talks')
        parser.add_argument('urlbase', metavar='URL_BASE',
            help='Prefix pad URLs with the given base')
        parser.add_argument('--suffix', default='.txt',
            help='Suffix pad URLs with the given extension. Default: .txt')

    def handle(self, *args, **options):
        if options['talk']:
            urls = TalkUrl.objects.filter(talk_id=options['talk'])
        else:
            urls = TalkUrl.objects.all()

        urls = urls.filter(description='etherpad')

        for talkurl in urls:
            m = re.search(r'/p/([^/]*)$', talkurl.url)
            if not m:
                print("Unable to parse pad URL:", talkurl.url)
                continue
            pad = m.group(1)
            talkurl.url = options['urlbase'] + pad + options['suffix']
            talkurl.save()



import datetime
import re
from uuid import UUID

from django.conf import settings
from django.core.management.base import BaseCommand

import requests

from wafer.schedule.models import ScheduleItem


class Command(BaseCommand):
    help = 'Load video URLs from sreview.debian.net'

    def add_arguments(self, parser):
        parser.add_argument(
            '--sreview-url', metavar='URL', type=str,
            default='https://sreview.debian.net/released.json',
            help='URL for the released.json file in the SReview instance',
        )

        current_year = datetime.datetime.today().year
        parser.add_argument(
            '--base-url', metavar='URL', type=str,
            default='https://meetings-archive.debian.net/pub/debian-meetings/%s' % current_year,
            help='Base URL for the videos released by sreview',
        )

        parser.add_argument(
            '--dry-run', action='store_true',
            help='Only show what would be done',
        )
        parser.add_argument(
            '--remove-lq', action='store_true',
            help='Strip .lq. from URLs',
        )

    def handle(self, *args, **options):
        jsonurl = options['sreview_url']

        jsondata = requests.get(jsonurl)
        jsondata.raise_for_status()

        data = jsondata.json()

        baseurl = options['base_url'].rstrip("/")
        scheduleitems_by_guid = {}
        scheduleitems_by_id = {}
        for scheduleitem in ScheduleItem.objects.all():
            scheduleitems_by_id[scheduleitem.id] = scheduleitem
            scheduleitems_by_guid[scheduleitem.guid] = scheduleitem

        slug_re = re.compile(
            re.escape(settings.WAFER_CONFERENCE_ACRONYM.lower())
            + r'-(\d+)-.*')

        for entry in data['videos']:
            m = slug_re.match(entry['eventid'])
            if m:
                talk_id = int(m.group(1))
                scheduleitem = scheduleitems_by_id.get(talk_id)
                if not scheduleitem:
                    print('ScheduleItem <%s> does not exist' % talk_id)
                    continue
            else:
                talk_guid = UUID(entry['eventid'])
                scheduleitem = scheduleitems_by_guid.get(talk_guid)
                if not scheduleitem:
                    print('ScheduleItem <%s> does not exist' % talk_guid)
                    continue
            talk = scheduleitem.talk
            if talk is None:
                print('ScheduleItem <%s> is not a talk' % talk_guid)
                continue
            url = baseurl + "/" + entry['video'].lstrip("/")
            if options['remove_lq']:
                parts = url.rpartition('.lq.')
                if '.' not in parts[2]:
                    url = parts[0] + '.' + parts[2]
            if options['dry_run']:
                print('Would load video for <%s>: %s' % (talk, url))
            else:
                talk.urls.update_or_create(
                    description='Video',
                    defaults={
                        "url": url,
                    }
                )
                print('Loaded video for <%s>: %s' % (talk, url))



from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand

from register.models import Attendee


class Command(BaseCommand):
    help = ("Automatically reconfirm attendees who have accepted a bursary or "
            "paid an invoice (and have final dates)")

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something')

    def check_reconfirm(self, attendee, dry_run):
        if attendee.billable() and attendee.paid():
            return self.reconfirm(attendee, dry_run)
        try:
            bursary = attendee.user.bursary
        except ObjectDoesNotExist:
            pass
        else:
            if bursary.status_in(None, ['accepted']):
                return self.reconfirm(attendee, dry_run)

    def reconfirm(self, attendee, dry_run):
        if dry_run:
            print('I would reconfirm:', attendee.user.username)
            return
        attendee.reconfirm = True
        attendee.save()

    def handle(self, *args, **options):
        if not settings.RECONFIRMATION:
            print('Set RECONFIRMATION=True and try again')
            return

        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')
        for attendee in Attendee.objects.filter(
                reconfirm=False, final_dates=True):
            if attendee.user.userprofile.is_registered():
                self.check_reconfirm(attendee, dry_run)



# Generated by Django 2.2.12 on 2020-05-30 18:35

import bursary.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bursary', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bursaryreferee',
            name='referee',
            field=models.ForeignKey(limit_choices_to=bursary.models.check_bursaryreferee_permission, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
    ]



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Layout


class Covid19Form(forms.Form):
    vaccinated = forms.BooleanField(
        label='I have been fully vaccinated against COVID-19',
        help_text='You are fully vaccinated if you have completed the full '
                  'course of vaccination, at least 2 weeks before arrival '
                  'at the conference venue.',
        required=False,
    )
    vaccination_notes = forms.CharField(
        label='Notes for the conference organisers',
        help_text='If you have not been vaccinated, please explain the '
                  'situation.',
        widget=forms.Textarea(attrs={'rows': 3}),
        required=False,
    )
    confirm_covid_tests = forms.BooleanField(
        label='I agree to be regularly tested for COVID-19',
        help_text='You agree to be regularly tested, as required by the '
                  '<a href="/about/covid19/">conference COVID-19 policy</a>.',
        required=False,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False
        self.helper.layout = Layout(
            Field('vaccinated', id='vaccinated'),
            Field('vaccination_notes'),
            Field('confirm_covid_tests', id='confirm_covid_tests'),
        )

    def clean(self):
        cleaned_data = super().clean()
        if (not cleaned_data.get('vaccinated')
                and not cleaned_data.get('vaccination_notes')):
            self.add_error(
                'vaccination_notes',
                'Please explain the reasons for not receiving a vaccine.')
        if (not cleaned_data.get('vaccinated')
                and not cleaned_data.get('confirm_covid_tests')):
            self.add_error(
                'confirm_covid_tests',
                'Please confirm your willingness to be regularly tested')

        return cleaned_data



from collections import OrderedDict

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.db.models import Q
from django.db.models.signals import pre_save, post_save
from django.dispatch import receiver
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.shortcuts import get_current_site

from debconf.tz import aoe_datetime
from invoices.models import Invoice


BURSARY_STATUS_CHOICES = (
    ('submitted', 'The bursary request has been submitted'),
    ('ranked', 'The bursary request has been ranked'),
    ('pending', 'The bursary request has been granted '
                'and is pending your acceptance'),
    ('accepted', 'The bursary has been accepted'),
    ('denied', 'The bursary has been denied'),
    ('expired', 'The bursary grant has expired'),
    ('canceled', 'The bursary request has been canceled'),
)

CONTRIB_RANKING_HELP_TEXT = """

Decimal value from 0 to 5; Please try to conform to the following scale for
your assessment.

<dl>
<dt>5 "Must fund"</dt>
<dd>
If this person does not attend DebConf, there will be significant
negative impact for DebConf or Debian more generally.
</dd>
<dt>4 "Priority funding"</dt>
<dd>
There are clear benefits to Debian or to DebConf of this person attending
debconf. This might be an accepted talk that seems particularly important.
</dd>
<dt>3 "Good initiative"</dt>
<dd>
We should fund this person because they propose something interesting.
</dd>
<dt>2 "Good record"</dt>
<dd>
This person has a record of substantial contribution to Debian.this should
generally be recent contribution, i.e. within the last two years.
</dd>
<dt>1 "OK"</dt>
<dd>
If we have budget, I don't object to funding this request.
</dd>
<dt>0 "Deny"</dt>
<dd>
Even if we have budget, I think we should not fund this request.
</dd>
</dl>
"""

OUTREACH_RANKING_HELP_TEXT = """

Decimal value from 0 to 5; Please try to assess applicants according to the
following criteria, and to use the full scale across your (non-null) ratings.

<ul>
<li>Gender (favoring women (cis, trans, queer), non-binary and genderqueer
individuals, as well as trans men)</li>
<li>Age (favoring individuals over 35 years old)</li>
<li>Country of origin / ethnic diversity</li>
<li>Whether the applicant is a newcomer to Debian (yes should rank higher)</li>
</ul>

"""


class Bursary(models.Model):
    BURSARY_NEEDS = OrderedDict((
        ('unable', 'Without this funding, I will be absolutely '
                   'unable to attend'),
        ('sacrifice', 'Without the requested funding, I will have to '
                      'make financial sacrifices to attend'),
        ('inconvenient', 'Without the requested funding, attending will '
                         'be inconvenient for me'),
        ('non-financial', 'I am not applying based on financial need'),
    ))
    CAN_UPDATE_STATUSES = ('submitted', 'ranked', 'pending')
    # Linked to User rather than Attendee, so we don't lose track if someone
    # unregisters
    user = models.OneToOneField(settings.AUTH_USER_MODEL,
                                related_name='bursary',
                                on_delete=models.PROTECT)

    # Request:
    request_food = models.BooleanField()
    request_accommodation = models.BooleanField()
    request_travel = models.BooleanField()
    request_expenses = models.BooleanField()

    reason_contribution = models.TextField(blank=True)
    reason_plans = models.TextField(blank=True)
    reason_diversity = models.TextField(blank=True)
    need = models.CharField(max_length=16,
                            choices=BURSARY_NEEDS.items(), blank=True)
    travel_bursary = models.IntegerField(null=True, blank=True)
    travel_from = models.TextField(blank=True)

    # Review:
    food_status = models.CharField(max_length=32,
                                   choices=BURSARY_STATUS_CHOICES,
                                   default='submitted')
    food_accept_before = models.DateField(null=True, blank=True)

    accommodation_status = models.CharField(max_length=32,
                                            choices=BURSARY_STATUS_CHOICES,
                                            default='submitted')
    accommodation_accept_before = models.DateField(null=True, blank=True)

    travel_status = models.CharField(max_length=32,
                                     choices=BURSARY_STATUS_CHOICES,
                                     default='submitted')
    travel_accept_before = models.DateField(null=True, blank=True)

    reimbursed_amount = models.IntegerField(default=0, blank=True)

    expenses_status = models.CharField(max_length=32,
                                       choices=BURSARY_STATUS_CHOICES,
                                       default='submitted')
    attendance_status = models.CharField(max_length=32,
                                         choices=BURSARY_STATUS_CHOICES,
                                         default='submitted')
    class Meta:
        verbose_name = 'bursary request'
        permissions = (
            ("review_attendance", "Can review attendance requests"),
        )

    @property
    def request_any(self):
        return (self.request_food or self.request_accommodation
                or self.request_travel or self.request_expenses)

    def potential_bursary(self, key=None):
        potential_bursary_status = frozenset((
            'submitted', 'ranked', 'pending', 'accepted'))
        if now() > aoe_datetime(settings.DEBCONF_BURSARY_ACCEPTANCE_DEADLINE):
            potential_bursary_status = frozenset(('accepted',))
        return self.status_in(key, potential_bursary_status)

    def can_update(self, key=None):
        ret = False
        if key is None or key == 'travel':
            ret = self.request_travel and self.travel_status == 'accepted'
        return ret or self.status_in(
            key=key, statuses=self.CAN_UPDATE_STATUSES
        )

    def status_in(self, key, statuses):
        """Return boolean result for whether a bursary for key has been
        requested and is in one of the listed statuses.
        If key is None, check all possible types of bursary.
        """

        if key == 'food':
            return (self.request_food
                    and self.food_status in statuses)
        elif key == 'accommodation':
            return (self.request_accommodation
                    and self.accommodation_status in statuses)
        elif key == 'travel':
            return (self.request_travel
                    and self.travel_status in statuses)
        elif key is None:
            return (self.status_in('food', statuses)
                    or self.status_in('travel', statuses)
                    or self.status_in('accommodation', statuses))
        else:
            raise ValueError('Unknown key for status_in %s' % key)

    def must_accept(self):
        return 'pending' in (self.food_status, self.accommodation_status,
                             self.travel_status)

    def notify_status(self, request):
        if not self.request_any:
            if self.attendance_status != 'submitted':
                self.notify_attendance(request)
            return

        from_email = 'bursaries@debconf.org'
        to = self.user.email
        subject = ("Your bursary request for %s: status updated" %
                   get_current_site(request).name)
        body = render_to_string(
            'bursary/notify_status.txt', {
                'object': self,
                'profile_url': request.build_absolute_uri(
                    reverse('wafer_user_profile', args=(self.user.username,))
                ),
            }, request=request)

        msg = EmailMultiAlternatives(subject, body, to=[to],
                                     from_email=from_email)

        msg.send()

    def notify_attendance(self, request):
        from_email = 'registration@debconf.org'
        to = self.user.email
        subject = ("Your registration request for %s: status updated" %
                   get_current_site(request).name)
        body = render_to_string(
            'bursary/notify_attendance_status.txt', {
                'object': self,
            }, request=request)

        msg = EmailMultiAlternatives(subject, body, to=[to],
                                     from_email=from_email)

        msg.send()

    def __str__(self):
        return 'Bursary <{}>'.format(self.user.username)


def validate_score(score):
    if not 0 <= score <= 5:
        raise ValidationError(
            _('%(score)s should be between 0 and 5 inclusive'),
            params={'score': score}
        )


def check_bursaryreferee_permission():
    # XXX olasd: this returns duplicates,
    #            and limit_choices_to can't do better :(

    content_type = ContentType.objects.get_for_model(BursaryReferee)
    permission = Permission.objects.get(codename='change_bursaryreferee',
                                        content_type=content_type)
    return (Q(groups__permissions=permission)
            | Q(user_permissions=permission))


class BursaryReferee(models.Model):
    bursary = models.ForeignKey(Bursary, related_name='referees',
                                on_delete=models.CASCADE)
    referee = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        limit_choices_to=check_bursaryreferee_permission,
    )

    contrib_score = models.DecimalField(
        null=True, verbose_name="Contribution score",
        help_text=CONTRIB_RANKING_HELP_TEXT,
        decimal_places=2,
        max_digits=3,
        validators=[validate_score],
        blank=True,
    )

    outreach_score = models.DecimalField(
        null=True, verbose_name="Diversity and Inclusion score",
        help_text=OUTREACH_RANKING_HELP_TEXT,
        decimal_places=2,
        max_digits=3,
        validators=[validate_score],
        blank=True,
    )

    notes = models.TextField(
        default='', verbose_name="Notes for evaluation",
        help_text="Let us know how you came to your decision",
        blank=True,
    )

    final = models.BooleanField(
        default=False, verbose_name="Final assessment",
    )

    class Meta:
        unique_together = ('bursary', 'referee')

    def __str__(self):
        referee_username = '???'
        if self.referee:
            referee_username = self.referee.username
        return 'BursaryReferee <{} ({}){}>'.format(
            self.bursary.user.username,
            referee_username,
            ' final' if self.final else '',
        )


@receiver(pre_save, sender=Bursary)
def auto_approve_attendance_on_bursary_acceptance(sender, instance, **kwargs):
    if instance.status_in(None, 'accepted'):
        instance.attendance_status = 'accepted'


@receiver(post_save, sender=Invoice)
def auto_approve_attendance_on_payment(sender, instance, **kwargs):
    try:
        recipient = instance.recipient
        bursary = recipient.bursary
        attendee = recipient.attendee
    except ObjectDoesNotExist:
        return
    if instance.status == 'paid' and attendee.paid():
        bursary.attendance_status = 'accepted'
        bursary.save()



from django.core.management.base import BaseCommand

from wafer.talks.models import ACCEPTED, Talk
from debconf.talk_urls import create_online_service_urls


class Command(BaseCommand):
    help = "Generate online service (Jitsi, etherpad, etc.) URLs for events"

    def add_arguments(self, parser):
        parser.add_argument('--regenerate', action='store_true',
            help='Delete existing URLs and replace them')
        parser.add_argument('--talk', type=int,
            help='Act only on a specific talk ID. Default: all ACCEPTED talks')

    def handle(self, *args, **options):
        if options['talk']:
            talks = Talk.objects.filter(talk_id=options['talk'])
        else:
            talks = Talk.objects.filter(status=ACCEPTED)

        for talk in talks:
            create_online_service_urls(talk, options['regenerate'])



from django.conf.urls import include
from django.urls import re_path


urlpatterns = [
    re_path(r'^badges/', include('badges.urls')),
    re_path(r'^bursary/', include('bursary.urls')),
    re_path(r'^front_desk/', include('front_desk.urls')),
    re_path(r'^invoices/', include('invoices.urls')),
    re_path(r'^register/', include('register.urls')),

    re_path(r'', include('debconf.urls')),
    re_path(r'', include('exports.urls')),
    re_path(r'', include('wafer.urls')),
]



from register.forms.visa import VisaForm
from register.models.visa import Visa
from register.views.core import RegisterStep


class VisaView(RegisterStep):
    template_name = 'register/page/visa.html'
    title = 'Visa'
    form_class = VisaForm

    def get_initial(self):
        user = self.request.user
        initial = {
            'country': user.attendee.country,
        }

        try:
            visa = user.attendee.visa
        except Visa.DoesNotExist:
            return initial

        initial['country'] = visa.country
        initial['visa'] = bool(visa.country)

        return initial

    def form_valid(self, form):
        attendee = self.request.user.attendee
        data = form.cleaned_data.copy()

        if not data.pop('visa'):
            Visa.objects.filter(attendee=attendee).delete()
            return super().form_valid(form)

        visa = Visa.objects.update_or_create(
            attendee=attendee, defaults=data)[0]
        attendee.visa = visa

        return super().form_valid(form)



# Generated by Django 4.2.8 on 2023-12-23 18:07

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("minidebconf", "0003_phone_registration_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="registration",
            name="phone_number",
            field=models.CharField(
                blank=True,
                max_length=24,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        inverse_match=True, regex="[^0-9() +-]"
                    )
                ],
            ),
        ),
    ]



from django.conf import settings

from bursary.models import Bursary
from register.forms.misc import MiscForm
from register.models.attendee import Attendee
from register.views.core import RegisterStep


class MiscView(RegisterStep):
    title = 'Anything Else?'
    form_class = MiscForm

    def get_initial(self):
        user = self.request.user
        initial = {
            'notes': self.request.user.attendee.notes
        }
        try:
            bursary = user.bursary
        except Bursary.DoesNotExist:
            pass
        else:
            for field in bursary._meta.get_fields():
                if field.is_relation:
                    continue
                initial[field.name] = getattr(bursary, field.name)
        return initial

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['contributions'] = self.request_contributions()
        return kwargs

    def request_contributions(self):
        user = self.request.user
        try:
            bursary = user.bursary
        except Bursary.DoesNotExist:
            bursary = Bursary()
        return (
            settings.DEBCONF_REVIEW_FREE_ATTENDEES
            and not user.attendee.billable()
            and not bursary.potential_bursary())

    def form_valid(self, form):
        user = self.request.user
        data = form.cleaned_data
        bursary_data = data.copy()
        attendee_data = {'notes': bursary_data.pop('notes')}
        bursary_data.update({
            'request_food': False,
            'request_accommodation': False,
            'request_travel': False,
            'request_expenses': False,
        })

        user.attendee  # We should never be creating, here
        user.attendee = Attendee.objects.update_or_create(
            user=user, defaults=attendee_data)[0]

        if self.request_contributions():
            user.bursary = Bursary.objects.update_or_create(
                user=user, defaults=bursary_data)[0]

        return super().form_valid(form)



import sys
import yaml

from django.conf import settings
from django.contrib.auth.models import Group
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand

from wafer.talks.models import TalkUrl
from wafer.schedule.models import ScheduleItem


FROM = 'content@debconf.org'

SUBJECT = '%(conference)s - upload URL for %(title)s'
BODY = '''\
Dear speaker,

We strongly recommend that you pre-record your talk
%(title)s

We will stream the talk, then follow it with a live Q&A session via Jitsi,
provided there is time.  Guidelines on how to record your talk can be found at
<https://debconf-video-team.pages.debian.net/docs/advice_for_recording.html>.
If you have any questions, please contact the content team on the
#debconf-content IRC channel on the OFTC network, or by email via
content@debconf.org.

When you are done with your recording, please upload it here:
%(url)s

If your session is not a "talk", but a BoF, or any other type of acvitity that
needs to happen live, just ignore this message.

Best regards,
The %(conference)s Content Team
'''


class Command(BaseCommand):
    help = "Notify speakers about their talks."

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),
        parser.add_argument('urls', type=open, metavar='URLFILE',
                            help='File with urls data (format: yaml dict with guid (from ScheduleItem) as key, url as value)'),

    def notify(self, talk, url, dry_run):
        kv, _ = talk.kv.get_or_create(
            group=self.content_group,
            key='notified_speaker_upload_url',
            defaults={'value': None},
        )

        if kv.value == url:
            return

        to = [user.email for user in talk.authors.all()]

        subst = {
            'title': talk.title,
            'conference': settings.DEBCONF_NAME,
            'url': url,
        }

        subject = SUBJECT % subst
        body = BODY % subst

        if dry_run:
            print('I would badger speakers of: %s' % talk.title)
            return
        else:
            print('Badgering speakers of: %s' % talk.title)
        email_message = EmailMultiAlternatives(
            subject, body, from_email=FROM, to=to)
        email_message.send()

        TalkUrl.objects.create(
            description="upload",
            url=url,
            talk=talk,
            public=False,
        )

        kv.value = url
        kv.save()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        self.content_group = Group.objects.get_by_natural_key('Talk Mentors')
        urls = yaml.load(options["urls"])
        for item in ScheduleItem.objects.all():
            if not item.talk:
                continue
            talk = item.talk
            url = urls.get(str(item.guid))
            if url:
                self.notify(talk, url, dry_run)
            else:
                print("W: no URL for \"%s\" found" % (talk.title))



# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.management.base import BaseCommand
from django.template.loader import get_template

from django.contrib.sites.models import Site

from invoices.models import Invoice


class Command(BaseCommand):
    help = 'Badger outstanding invoices'

    def add_arguments(self, parser):
        parser.add_argument('--yes', action='store_true',
                            help='Actually do something'),

    def badger(self, invoice, dry_run):
        to = invoice.recipient.email
        name = invoice.recipient.get_full_name()
        invoice_number = invoice.reference_number
        total = invoice.total
        currency = settings.DEBCONF_BILLING_CURRENCY
        site = Site.objects.first()

        if not total:
            return

        if dry_run:
            print(f'I would badger {name} <{to}> '
                  f'({invoice_number}: {total} {currency})')
            return

        ctx = {
            'name': name,
            'total': total,
            'currency': currency,
            'invoice_number': invoice_number,
            'invoice_details': invoice.text_details(),
            'WAFER_CONFERENCE_NAME': site.name,
            'WAFER_CONFERENCE_DOMAIN': site.domain,
        }

        template = get_template('invoices/outstanding_invoices-subject.txt')
        subject = template.render(ctx).strip()

        template = get_template('invoices/outstanding_invoices-body.txt')
        body = template.render(ctx)

        email_message = EmailMultiAlternatives(subject, body,
                                               to=["%s <%s>" % (name, to)])
        email_message.send()

    def handle(self, *args, **options):
        dry_run = not options['yes']
        if dry_run:
            print('Not actually doing anything without --yes')

        for invoice in Invoice.objects.filter(status='new'):
            if invoice.recipient.userprofile.is_registered():
                self.badger(invoice, dry_run)



from django import forms
from django.conf import settings
from django.utils import formats
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy as _p

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Submit, HTML

from minidebconf.models import Diet, Registration, RegistrationType, ShirtSize
from minidebconf.models import ScheduleBlock

def format_days():
    days = []
    for day in ScheduleBlock.objects.all():
        date = formats.date_format(day.start_time, format="SHORT_DATE_FORMAT")
        weekday = formats.date_format(day.start_time, format="l")
        days.append((day.pk, f"{date} ({weekday})"))
    return days

def register_form_factory():
    form_fields = ['full_name']
    notes = False
    if settings.MINIDEBCONF_REGISTER_PHONE is not None:
        form_fields.append('phone_number')
    if RegistrationType.objects.exists():
        form_fields.append('registration_type')
    form_fields.append('full_name')
    form_fields.append('involvement')
    form_fields.append('gender')
    form_fields.append('country')
    form_fields.append('city_state')
    if ScheduleBlock.objects.count() > 0:
        form_fields.append('days')
    if settings.MINIDEBCONF_REGISTER_ARRANGED_ACCOMMODATION:
        form_fields.append('arranged_accommodation')
        notes = True
    if settings.MINIDEBCONF_REGISTER_ARRANGED_FOOD:
        form_fields.append('arranged_food')
    if Diet.objects.exists():
        form_fields.append('diet')
    if settings.MINIDEBCONF_REGISTER_TRAVEL_REIMBURSEMENT:
        form_fields.append("travel_reimbursement")
        form_fields.append("travel_cost")
        notes = True
    if notes:
        form_fields.append("notes")
    if ShirtSize.objects.exists():
        form_fields.append('shirt_size')


    class RegisterForm(forms.ModelForm):
        full_name = forms.CharField(max_length=256, label=_('Full name'))
        class Meta:
            model = Registration
            fields = form_fields
            widgets = {
                'days': forms.CheckboxSelectMultiple(),
            }

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.fields["full_name"].initial = self.instance.full_name
            if 'days' in self.fields:
                self.fields["days"].choices = format_days()
            if 'phone_number' in self.fields:
                self.fields["phone_number"].required = settings.MINIDEBCONF_REGISTER_PHONE
            if settings.MINIDEBCONF_REGISTER_DEFAULT_COUNTRY:
                self.initial['country'] = settings.MINIDEBCONF_REGISTER_DEFAULT_COUNTRY
            if 'arranged_accommodation' in self.fields:
                self.fields['arranged_accommodation'].label = _('I would like to stay at the conference-arranged accommodation')
            if 'arranged_food' in self.fields:
                self.fields['arranged_food'].label = _('I would like to have the conference-arranged meals')
            if 'travel_reimbursement' in self.fields:
                self.fields["travel_reimbursement"].label = _('I would like to request reimbursement of my travel costs')
            if 'travel_cost' in self.fields:
                self.fields['travel_cost'].label = _('Estimated travel cost (upper bound, in %(currency)s)') % { 'currency': settings.DEBCONF_BURSARY_CURRENCY_SYMBOL }

            self.helper = FormHelper()

            layout = [
                'full_name',
                'phone_number',
                'involvement',
                'gender',
                'country',
                'city_state',
                'days',
            ]
            if 'shirt_size' in self.fields:
                if page := settings.MINIDEBCONF_REGISTER_SHIRT_INFO_PAGE:
                    info = [HTML(_("<p><a href='%s' target='_blank'>See this page</a> for more information about shirt sizes.</p>") % page)]
                else:
                    info = []
                layout += [
                    Fieldset(
                        _('T-Shirt'),
                        *info,
                        'shirt_size',
                    )
                ]
            if 'diet' in self.fields or 'arranged_food' in self.fields:
                layout += [
                    Fieldset(
                        _('Food'),
                        'arranged_food',
                        'diet',
                    )
                ]
            if 'arranged_accommodation' in self.fields or 'travel_reimbursement' in self.fields:
                if page := settings.MINIDEBCONF_REGISTER_BURSARY_INFO_PAGE:
                    info = [HTML(_("<p><a href='%s' target='_blank'>See this page</a> for more information about bursaries.</p>") % page)]
                else:
                    info = []
                layout += [
                    Fieldset(
                        _('Bursaries'),
                        *info,
                        'arranged_accommodation',
                        'travel_reimbursement',
                        'travel_cost',
                        'notes',
                        id='bursary_fields',
                    )
                ]

            if self.instance.id:
                submit = _p("conference", "Update registration")
            else:
                submit = _p("conference", "Register")
            layout += [Submit("submit", submit)]
            self.helper.layout = Layout(*layout)


        def save(self):
            super().save()
            name = self.cleaned_data['full_name'].split()
            user = self.instance.user
            user.first_name = name[0]
            user.last_name = " ".join(name[1:])
            user.save()
    return RegisterForm



from datetime import datetime, time, timedelta
from django.core.management.base import BaseCommand
from django.utils.timezone import make_aware

from wafer.pages.models import Page
from wafer.schedule.models import ScheduleBlock, Slot, Venue, ScheduleItem

import yaml


def parse_time(s):
    h, m = s.split(':')
    return time(int(h), int(m))


class Command(BaseCommand):
    help = 'Load Schedule days, venues, and slots from YAML files into the DB'

    def add_arguments(self, parser):
        parser.add_argument('YAMLFILE', help='Input file'),

    def handle(self, *args, **options):
        with open(options['YAMLFILE']) as f:
            data = yaml.safe_load(f)

        schedule_blocks = {}
        start_of_day = time(0, 0, 0)
        end_of_day = time(23, 59, 59)
        for day in data['days']:
            schedule_block, _ = ScheduleBlock.objects.get_or_create(
                start_time=make_aware(
                    datetime.combine(day, start_of_day) + timedelta(hours=3)),
                end_time=make_aware(
                    datetime.combine(day, end_of_day) + timedelta(hours=3)),
            )
            schedule_blocks[day] = schedule_block
            print('Loaded schedule block: %s' % schedule_block)

        venues = []
        for v in data['venues']:
            venue_name = v['name']
            venue, _ = Venue.objects.get_or_create(name=venue_name)
            venue.notes = v['notes']
            venue.order = v['order']
            venue.video = v.get("video", False)
            venue_blocks = [day for d, day in schedule_blocks.items()
                            if d in v['days']]
            venue.blocks.set(venue_blocks)
            venue.save()
            venues.append(venue)
            print('Loaded venue: %s (%r)' % (venue, venue_blocks))

        for day, slots in data['slots'].items():
            schedule_block = schedule_blocks[day]
            sb_date = schedule_block.start_time.date()
            for slotdata in slots:
                slot, _ = Slot.objects.get_or_create(
                    name='{}: {}'.format(slotdata['name'], sb_date))
                start_time = parse_time(slotdata['start'])
                slot.start_time = make_aware(datetime.combine(sb_date, start_time))
                end_time = parse_time(slotdata['end'])
                slot.end_time = make_aware(datetime.combine(sb_date, end_time))
                if slotdata.get('nextday'):
                    slot.start_time = slot.start_time + timedelta(days=1)
                    slot.end_time = slot.end_time + timedelta(days=1)
                slot.save()
                print('Loaded slot: %s' % slot)

                if slotdata.get('break', False):
                    schedule, _ = Page.objects.get_or_create(
                        name='Schedule',
                        slug='schedule',
                    )
                    elsewhere, _ = Venue.objects.get_or_create(name="Elsewhere")
                    page, _ = Page.objects.get_or_create(
                        name=slotdata['name'],
                        slug=slotdata['name'].lower().replace(' ', '-'),
                        parent=schedule,
                    )
                    if not page.content:
                        page.content = '# ' + slotdata['name'] + "\n"
                        page.save()

                    try:
                        ScheduleItem.objects.get(
                            page=page,
                            venue=elsewhere,
                            expand=True,
                            slots=slot,
                        )
                    except ScheduleItem.DoesNotExist:
                        item = ScheduleItem.objects.create(
                            page=page,
                            venue=elsewhere,
                            expand=True,
                        )
                        item.slots.add(slot)



from django.urls import re_path

from invoices.views import (
    InvoiceCancel, InvoiceCombine, InvoiceDisplay, stripe_webhook)

app_name = 'invoices'
urlpatterns = [
    re_path(r'^combine/$', InvoiceCombine.as_view(), name='combine'),
    re_path(r'^stripe-webhook/$', stripe_webhook),
    re_path(r'^(?P<reference_number>[^/]+)/$', InvoiceDisplay.as_view(),
        name='display'),
    re_path(r'^(?P<reference_number>[^/]+)/cancel/$', InvoiceCancel.as_view(),
        name='cancel'),
]



from datetime import timedelta


def get_start_end_for_scheduleitem(si):
    if not si.slots.exists():
        raise ValueError('M2M not realized yet, come back later')
    start = si.get_start_datetime()
    end = start + timedelta(minutes=si.get_duration_minutes())
    return (start, end)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-04 19:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('register', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CheckIn',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('t_shirt', models.BooleanField(default=False)),
                ('shoes', models.BooleanField(default=False)),
                ('swag', models.BooleanField(default=False)),
                ('nametag', models.BooleanField(default=False)),
                ('key_card', models.CharField(blank=True, max_length=5)),
                ('room_key', models.BooleanField(default=False)),
                ('checked_out', models.BooleanField(default=False)),
                ('returned_key', models.BooleanField(default=False)),
                ('returned_card', models.BooleanField(default=False)),
                ('notes', models.TextField(blank=True)),
                ('attendee', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='check_in', to='register.Attendee')),
            ],
        ),
    ]



from django import forms

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Fieldset, Layout


class MiscForm(forms.Form):
    reason_contribution = forms.CharField(
        label='My contributions to Debian',
        widget=forms.Textarea(attrs={'rows': 5}),
        required=False,
        help_text='To help us evaluate your registration.',
    )
    reason_plans = forms.CharField(
        label='My plans for DebCamp or DebConf',
        help_text='To help us evaluate your registration.',
        widget=forms.Textarea(attrs={'rows': 5}),
        required=False,
    )
    notes = forms.CharField(
        label='Special Needs and Notes for the registration team',
        help_text='Anything else you need to describe. '
                  'The registration team will see this. '
                  'The bursaries team will not.',
        widget=forms.Textarea(attrs={'rows': 3}),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        self.contributions = kwargs.pop('contributions')
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.include_media = False

        reasons = ()
        if self.contributions:
            reasons = (
                Fieldset(
                    'Contributions to Debian',
                    Field('reason_contribution'),
                    Field('reason_plans'),
                    css_id='contributions',
                ),
            )
        self.helper.layout = Layout(
            Field('notes'),
            *reasons
        )

    def clean(self):
        cleaned_data = super().clean()

        if self.contributions:
            if not cleaned_data.get('reason_plans'):
                self.add_error(
                    'reason_plans',
                    'Please share your plans for the conference, when applying '
                    'for free attendance.')

            if not cleaned_data.get('reason_contribution'):
                self.add_error(
                    'reason_contribution',
                    'Please describe your contributions and/or the '
                    'diversity of your background, when applying for free '
                    'attendance.')



from django.contrib import admin
from django.utils.dateparse import parse_date

from register.admin.core import NotNullFilter
from register.models.attendee import Attendee
from register.dates import LEAVE_ON_OR_BEFORE, nights


class NotesNotNullFilter(NotNullFilter):
    title = "Notes"
    parameter_name = "notes"


class DateFilter(admin.SimpleListFilter):
    def lookups(self, request, model_admin):
        for day in nights(orga=True):
            yield day.isoformat(), day
        yield LEAVE_ON_OR_BEFORE.isoformat(), LEAVE_ON_OR_BEFORE
        yield 'null', 'None'

    def queryset(self, request, queryset):
        parameter = self.parameter_name
        value = self.value()
        if value is None:
            return queryset
        if value == 'null':
            parameter += '__isnull'
            value = True
        else:
            parameter += '__date'
            value = parse_date(value)
        return queryset.filter(**{parameter: value})


class ArrivalDateFilter(DateFilter):
    title = 'Arrival Day'
    parameter_name = 'arrival'


class DepartureDateFilter(DateFilter):
    title = 'Departure Day'
    parameter_name = 'departure'


class AttendeeAdmin(admin.ModelAdmin):
    list_display = (
        'user', 'full_name', 'email', 'reconfirm', 'arrival', 'departure',
        'billable', 'paid', 'registration_approved',
    )
    list_filter = (
        'fee', ArrivalDateFilter, DepartureDateFilter,
        'reconfirm', 't_shirt_size', 'shoe_size', NotesNotNullFilter,
    )
    search_fields = ('user__username', 'user__first_name', 'user__last_name')
    readonly_fields = ('full_name', 'email', 'shipping_address')

    def full_name(self, instance):
        return instance.user.get_full_name()
    full_name.admin_order_field = 'user__last_name'

    def email(self, instance):
        return instance.user.email
    email.admin_order_field = 'user__email'

    def shipping_address(self, instance):
        return instance.shipping_address.formatted_address


admin.site.register(Attendee, AttendeeAdmin)



from django.utils import timezone

from pytest import fixture

from debconf import settings as dc_settings


@fixture(autouse=True)
def debconf_settings(settings):
    for k, v in dc_settings.__dict__.items():
        if k.isupper() and k not in {'DATABASES',}:
            setattr(settings, k, v)
    settings.ROOT_URLCONF = 'tests.debconf.urls'
    settings.WAFER_USER_IS_REGISTERED = 'register.models.user_is_registered'


@fixture
def attendee(db, user):
    from register.models.attendee import Attendee
    attendee = Attendee.objects.create(
        user=user,
        nametag_2='Line 2',
        nametag_3='Line 3',
        emergency_contact='Emergency Contact <contact@example.com>',
        announce_me=True,
        register_announce=False,
        register_discuss=False,
        coc_ack=True,
        fee='',
        arrival=None,
        departure=None,
        final_dates=False,
        reconfirm=True,
        t_shirt_size='l',
        shoe_size='l',
        gender='n',
        country='CA',
        languages='Python',
        pgp_fingerprints='',
        invoiced_entity='',
        billing_address='Billing Address',
        notes='',
        completed_register_steps=13,
        completed_timestamp=timezone.now(),
    )
    yield attendee
    attendee.delete()


@fixture
def bursary(db, attendee):
    from bursary.models import Bursary
    bursary = Bursary.objects.create(
        user=attendee.user,
        request_travel=True,
        request_food=True,
        request_accommodation=True,
        request_expenses=False
    )
    yield bursary
    bursary.delete()



from django.db import models

from register.models.attendee import Attendee


class ChildCare(models.Model):
    attendee = models.OneToOneField(Attendee, related_name='child_care',
                                    on_delete=models.CASCADE)

    needs = models.TextField(blank=True)
    details = models.TextField(blank=True)

    def __str__(self):
        return 'ChildCare <{}>'.format(self.attendee.user.username)
